@@ -2,7 +2,7 @@
 /* *********************************************************************** */
 /*                                                                         */
 /* Library :  Generic Matrix Methods  (gmm)                                */
-/* File    :  gmm_transposed.h : generic transposed matrices.              */
+/* File    :  gmm_scaled.h : generic scaled vectors and matrices.          */
 /*     									   */
 /* Date : November 10, 2002.                                               */
 /* Author : Yves Renard, Yves.Renard@gmm.insa-tlse.fr                      */
@@ -11,7 +11,7 @@
 /*                                                                         */
 /* Copyright (C) 2002  Yves Renard.                                        */
 /*                                                                         */
-/* This file is a part of GETFEM++                                         */
+/* This file is a part of GMM++                                            */
 /*                                                                         */
 /* This program is free software; you can redistribute it and/or modify    */
 /* it under the terms of the GNU Lesser General Public License as          */
@@ -29,11 +29,10 @@
 /*                                                                         */
 /* *********************************************************************** */
 
-//   . faire scale et scaled sur les matrices aussi
-//
+#ifndef GMM_SCALED_H__
+#define GMM_SCALED_H__
 
-#ifndef __GMM_SCALED_H
-#define __GMM_SCALED_H
+#include <gmm_def.h>
 
 namespace gmm {
 
@@ -41,7 +40,7 @@ namespace gmm {
   /*		Scaled references on vectors            		   */
   /* ********************************************************************* */
 
-  template <class IT> struct scaled_const_iterator {
+  template <typename IT> struct scaled_const_iterator {
     typedef typename std::iterator_traits<IT>::value_type      value_type;
     typedef typename std::iterator_traits<IT>::pointer         pointer;
     typedef typename std::iterator_traits<IT>::reference       reference;
@@ -56,105 +55,90 @@ namespace gmm {
     scaled_const_iterator(const IT &i, value_type x) : it(i), r(x) {}
     
     inline size_type index(void) const { return it.index(); }
-    scaled_const_iterator operator ++(int)
+    inline scaled_const_iterator operator ++(int)
     { scaled_const_iterator tmp = *this; ++it; return tmp; }
-    scaled_const_iterator operator --(int) 
+    inline scaled_const_iterator operator --(int) 
     { scaled_const_iterator tmp = *this; --it; return tmp; }
-    scaled_const_iterator &operator ++() { ++it; return *this; }
-    scaled_const_iterator &operator --() { --it; return *this; }
-    scaled_const_iterator &operator +=(difference_type i)
+    inline scaled_const_iterator &operator ++() { ++it; return *this; }
+    inline scaled_const_iterator &operator --() { --it; return *this; }
+    inline scaled_const_iterator &operator +=(difference_type i)
       { it += i; return *this; }
-    scaled_const_iterator &operator -=(difference_type i)
+    inline scaled_const_iterator &operator -=(difference_type i)
       { it -= i; return *this; }
-    scaled_const_iterator operator +(difference_type i) const
+    inline scaled_const_iterator operator +(difference_type i) const
       { scaled_const_iterator itb = *this; return (itb += i); }
-    scaled_const_iterator operator -(difference_type i) const
+    inline scaled_const_iterator operator -(difference_type i) const
       { scaled_const_iterator itb = *this; return (itb -= i); }
-    difference_type operator -(const scaled_const_iterator &i) const
+    inline difference_type operator -(const scaled_const_iterator &i) const
       { return difference_type(it - i.it); }
     
-    value_type operator  *() const { return (*it) * r; }
-    value_type operator [](size_type ii) const { return it[ii] * r; }
+    inline value_type operator  *() const { return (*it) * r; }
+    inline value_type operator [](size_type ii) const { return it[ii] * r; }
     
-    bool operator ==(const scaled_const_iterator &i) const
+    inline bool operator ==(const scaled_const_iterator &i) const
       { return (i.it == it); }
-    bool operator !=(const scaled_const_iterator &i) const
+    inline bool operator !=(const scaled_const_iterator &i) const
       { return (i.it != it); }
-    bool operator < (const scaled_const_iterator &i) const
+    inline bool operator < (const scaled_const_iterator &i) const
       { return (it < i.it); }
   };
 
-  template <class V> struct scaled_vector_const_ref {
+  template <typename V> struct scaled_vector_const_ref {
     typedef scaled_vector_const_ref<V> this_type;
     typedef typename linalg_traits<V>::value_type value_type;
     typedef typename linalg_traits<V>::const_iterator iterator;
     typedef typename linalg_traits<this_type>::reference reference;
-    typedef typename linalg_traits<V>::access_type access_type;
+    typedef typename linalg_traits<this_type>::origin_type origin_type;
 
-    iterator _begin, _end;
-    const void *origin;
-    size_type _size;
+    iterator begin_, end_;
+    const origin_type *origin;
+    size_type size_;
     value_type r;
 
     scaled_vector_const_ref(const V &v, value_type rr)
-      : _begin(vect_const_begin(v)), _end(vect_const_end(v)), origin(linalg_origin(v)),
-	_size(vect_size(v)), r(rr) {}
+      : begin_(vect_const_begin(v)), end_(vect_const_end(v)),
+	origin(linalg_origin(v)), size_(vect_size(v)), r(rr) {}
 
     reference operator[](size_type i) const
-    { return r * access_type()(origin, _begin, _end, i); }
+    { return r * linalg_traits<V>::access(origin, begin_, end_, i); }
   };
 
-  template <class V> struct scaled_vector_const_access {
-    typedef scaled_vector_const_ref<V> this_type;
-    typedef typename linalg_traits<V>::value_type value_type;
-    typedef typename linalg_traits<this_type>::const_iterator iterator;
-    
-    value_type operator()(const void *o, const iterator &_begin,
-			  const iterator &_end, size_type i) {
-      return _begin.r 
-	* typename linalg_traits<V>::access_type()(o, _begin.it, _end.it, i);
-    }
-  };
-
-  template <class V> struct linalg_traits<scaled_vector_const_ref<V> > {
+  template <typename V> struct linalg_traits<scaled_vector_const_ref<V> > {
     typedef scaled_vector_const_ref<V> this_type;
     typedef linalg_const is_reference;
     typedef abstract_vector linalg_type;
     typedef typename linalg_traits<V>::value_type value_type;
+    typedef typename linalg_traits<V>::origin_type origin_type;
     typedef value_type reference;
     typedef abstract_null_type iterator;
     typedef scaled_const_iterator<typename linalg_traits<V>::const_iterator>
             const_iterator;
     typedef typename linalg_traits<V>::storage_type storage_type;
-    typedef scaled_vector_const_access<V> access_type;
-    typedef abstract_null_type clear_type;
-    static size_type size(const this_type &v) { return v._size; }
+    static size_type size(const this_type &v) { return v.size_; }
     static iterator begin(this_type &v)
-    { return iterator(v._begin, v.r); }
+    { return iterator(v.begin_, v.r); }
     static const_iterator begin(const this_type &v)
-    { return const_iterator(v._begin, v.r); }
+    { return const_iterator(v.begin_, v.r); }
     static iterator end(this_type &v)
-    { return iterator(v._end, v.r); }
+    { return iterator(v.end_, v.r); }
     static const_iterator end(const this_type &v)
-    { return const_iterator(v._end, v.r); }
-    static const void* origin(const this_type &v) { return v.origin; }
+    { return const_iterator(v.end_, v.r); }
+    static const origin_type* origin(const this_type &v) { return v.origin; }
+    static value_type access(const origin_type *o, const const_iterator &it,
+			     const const_iterator &ite, size_type i)
+    { return it.r * (linalg_traits<V>::access(o, it.it, ite.it, i)); }
+
   };
 
-   template<class V> std::ostream &operator <<
+   template<typename V> std::ostream &operator <<
   (std::ostream &o, const scaled_vector_const_ref<V>& m)
   { gmm::write(o,m); return o; }
 
-#ifdef USING_BROKEN_GCC295
-  template <class V> struct linalg_traits<const scaled_vector_const_ref<V> > 
-    : public linalg_traits<scaled_vector_const_ref<V> > {};
-#endif
-
-
   /* ********************************************************************* */
   /*		Scaled references on matrices            		   */
   /* ********************************************************************* */
 
-  template <class M> struct scaled_row_const_iterator {
+  template <typename M> struct scaled_row_const_iterator {
     typedef scaled_row_const_iterator<M> iterator;
     typedef typename linalg_traits<M>::const_row_iterator ITER;
     typedef typename linalg_traits<M>::value_type value_type;
@@ -164,10 +148,10 @@ namespace gmm {
     ITER it;
     value_type r;
 
-    iterator operator ++(int) { iterator tmp = *this; it++; return tmp; }
-    iterator operator --(int) { iterator tmp = *this; it--; return tmp; }
-    iterator &operator ++()   { it++; return *this; }
-    iterator &operator --()   { it--; return *this; }
+    inline iterator operator ++(int) { iterator tmp=*this; it++; return tmp; }
+    inline iterator operator --(int) { iterator tmp=*this; it--; return tmp; }
+    inline iterator &operator ++()   { it++; return *this; }
+    inline iterator &operator --()   { it--; return *this; }
     iterator &operator +=(difference_type i) { it += i; return *this; }
     iterator &operator -=(difference_type i) { it -= i; return *this; }
     iterator operator +(difference_type i) const 
@@ -177,12 +161,12 @@ namespace gmm {
     difference_type operator -(const iterator &i) const
     { return it - i.it; }
 
-    ITER operator *() const { return it; }
-    ITER operator [](int i) { return it + i; }
+    inline ITER operator *() const { return it; }
+    inline ITER operator [](int i) { return it + i; }
 
-    bool operator ==(const iterator &i) const { return (it == i.it); }
-    bool operator !=(const iterator &i) const { return !(i == *this); }
-    bool operator < (const iterator &i) const { return (it < i.it); }
+    inline bool operator ==(const iterator &i) const { return (it == i.it); }
+    inline bool operator !=(const iterator &i) const { return !(i == *this); }
+    inline bool operator < (const iterator &i) const { return (it < i.it); }
 
     scaled_row_const_iterator(void) {}
     scaled_row_const_iterator(const ITER &i, value_type rr)
@@ -190,39 +174,31 @@ namespace gmm {
 
   };
 
-  template <class M> struct  scaled_row_matrix_const_ref {
+  template <typename M> struct  scaled_row_matrix_const_ref {
     
     typedef scaled_row_matrix_const_ref<M> this_type;
     typedef typename linalg_traits<M>::const_row_iterator iterator;
     typedef typename linalg_traits<M>::value_type value_type;
-    typedef typename linalg_traits<M>::access_type access_type;
+    typedef typename linalg_traits<this_type>::origin_type origin_type;
 
-    iterator _begin, _end;
-    const void *origin;
+    iterator begin_, end_;
+    const origin_type *origin;
     value_type r;
+    size_type nr, nc;
 
     scaled_row_matrix_const_ref(const M &m, value_type rr)
-      : _begin(mat_row_begin(m)), 
-      _end(mat_row_end(m)), origin(linalg_origin(m)), r(rr) {}
+      : begin_(mat_row_begin(m)), end_(mat_row_end(m)),
+	origin(linalg_origin(m)), r(rr), nr(mat_nrows(m)), nc(mat_ncols(m)) {}
 
     value_type operator()(size_type i, size_type j) const
-    { return r * access_type()(_begin+i, j); }
+    { return r * linalg_traits<M>::access(begin_+i, j); }
   };
 
-  template <class M> struct scaled_row_matrix_access {
-    typedef scaled_row_matrix_const_ref<M> this_type;
-    typedef typename linalg_traits<this_type>::row_iterator iterator;
-    typedef typename linalg_traits<M>::value_type value_type;
-    typedef typename linalg_traits<M>::access_type access_type;
-    
-    value_type operator()(const iterator &itrow, size_type i)
-    { return itrow.r * access_type(itrow.it, i); }
-  };
-
-  template <class M> struct linalg_traits<scaled_row_matrix_const_ref<M> > {
+  template <typename M> struct linalg_traits<scaled_row_matrix_const_ref<M> > {
     typedef scaled_row_matrix_const_ref<M> this_type;
     typedef linalg_const is_reference;
     typedef abstract_matrix linalg_type;
+    typedef typename linalg_traits<M>::origin_type origin_type;
     typedef typename linalg_traits<M>::value_type value_type;
     typedef value_type reference;
     typedef typename linalg_traits<M>::storage_type storage_type;
@@ -236,31 +212,27 @@ namespace gmm {
     typedef abstract_null_type const_col_iterator;
     typedef abstract_null_type col_iterator;
     typedef row_major sub_orientation;
-    typedef scaled_row_matrix_access<M> access_type;
-    static size_type nrows(const this_type &m) { return m._end - m._begin; }
+    static size_type nrows(const this_type &m)
+    { return m.nr; }
     static size_type ncols(const this_type &m)
-    { return (nrows(m) == 0) ? 0 : vect_size(mat_row(m, 0)); }
+    { return m.nc; }
     static const_sub_row_type row(const const_row_iterator &it)
     { return scaled(linalg_traits<M>::row(it.it), it.r); }
     static const_row_iterator row_begin(const this_type &m)
-    { return const_row_iterator(m._begin, m.r); }
+    { return const_row_iterator(m.begin_, m.r); }
     static const_row_iterator row_end(const this_type &m)
-    { return const_row_iterator(m._end, m.r); }
-    static const void* origin(const this_type &m) { return m.origin; }
+    { return const_row_iterator(m.end_, m.r); }
+    static const origin_type* origin(const this_type &m) { return m.origin; }
+    static value_type access(const const_row_iterator &it, size_type i)
+    { return it.r * (linalg_traits<M>::access(it.it, i)); }
   };
 
-  template<class M> std::ostream &operator <<
+  template<typename M> std::ostream &operator <<
   (std::ostream &o, const scaled_row_matrix_const_ref<M>& m)
   { gmm::write(o,m); return o; }
 
-#ifdef USING_BROKEN_GCC295
-  template <class M>
-  struct linalg_traits<const scaled_row_matrix_const_ref<M> > 
-    : public linalg_traits<scaled_row_matrix_const_ref<M> > {};
-#endif
 
-
-  template <class M> struct scaled_col_const_iterator {
+  template <typename M> struct scaled_col_const_iterator {
     typedef scaled_col_const_iterator<M> iterator;
     typedef typename linalg_traits<M>::const_col_iterator ITER;
     typedef typename linalg_traits<M>::value_type value_type;
@@ -296,40 +268,32 @@ namespace gmm {
 
   };
 
-  template <class M> struct  scaled_col_matrix_const_ref {
+  template <typename M> struct  scaled_col_matrix_const_ref {
     
     typedef scaled_col_matrix_const_ref<M> this_type;
     typedef typename linalg_traits<M>::const_col_iterator iterator;
     typedef typename linalg_traits<M>::value_type value_type;
-    typedef typename linalg_traits<M>::access_type access_type;
+    typedef typename linalg_traits<this_type>::origin_type origin_type;
 
-    iterator _begin, _end;
-    const void *origin;
+    iterator begin_, end_;
+    const origin_type *origin;
     value_type r;
+    size_type nr, nc;
 
     scaled_col_matrix_const_ref(const M &m, value_type rr)
-      : _begin(mat_col_begin(m)), 
-      _end(mat_col_end(m)), origin(linalg_origin(m)), r(rr) {}
+      : begin_(mat_col_begin(m)), end_(mat_col_end(m)),
+	origin(linalg_origin(m)), r(rr), nr(mat_nrows(m)), nc(mat_ncols(m)) {}
 
     value_type operator()(size_type i, size_type j) const
-    { return r * access_type()(_begin+i, j); }
+    { return r * linalg_traits<M>::access(begin_+j, i); }
   };
 
-  template <class M> struct scaled_col_matrix_access {
-    typedef scaled_col_matrix_const_ref<M> this_type;
-    typedef typename linalg_traits<this_type>::const_col_iterator iterator;
-    typedef typename linalg_traits<M>::value_type value_type;
-    typedef typename linalg_traits<M>::access_type access_type;
-    
-    value_type operator()(const iterator &itcol, size_type i)
-    { return itcol.r * access_type(itcol.it, i); }
-  };
-
-  template <class M> struct linalg_traits<scaled_col_matrix_const_ref<M> > {
+  template <typename M> struct linalg_traits<scaled_col_matrix_const_ref<M> > {
     typedef scaled_col_matrix_const_ref<M> this_type;
     typedef linalg_const is_reference;
     typedef abstract_matrix linalg_type;
     typedef typename linalg_traits<M>::value_type value_type;
+    typedef typename linalg_traits<M>::origin_type origin_type;
     typedef value_type reference;
     typedef typename linalg_traits<M>::storage_type storage_type;
     typedef typename linalg_traits<M>::const_sub_col_type vector_type;
@@ -342,79 +306,75 @@ namespace gmm {
     typedef abstract_null_type const_row_iterator;
     typedef abstract_null_type row_iterator;
     typedef col_major sub_orientation;
-    typedef scaled_col_matrix_access<M> access_type;
-    static size_type ncols(const this_type &m) { return m._end - m._begin; }
+    static size_type ncols(const this_type &m)
+    { return m.nc; }
     static size_type nrows(const this_type &m)
-    { return (ncols(m) == 0) ? 0 : vect_size(mat_col(m, 0)); }
+    { return m.nr; }
     static const_sub_col_type col(const const_col_iterator &it)
     { return scaled(linalg_traits<M>::col(it.it), it.r); }
     static const_col_iterator col_begin(const this_type &m)
-    { return const_col_iterator(m._begin, m.r); }
+    { return const_col_iterator(m.begin_, m.r); }
     static const_col_iterator col_end(const this_type &m)
-    { return const_col_iterator(m._end, m.r); }
-    static const void* origin(const this_type &m) { return m.origin; }
+    { return const_col_iterator(m.end_, m.r); }
+    static const origin_type* origin(const this_type &m) { return m.origin; }
+    static value_type access(const const_col_iterator &it, size_type i)
+    { return it.r * (linalg_traits<M>::access(it.it, i)); }
   };
 
-  template<class M> std::ostream &operator <<
+  template<typename M> std::ostream &operator <<
   (std::ostream &o, const scaled_col_matrix_const_ref<M>& m)
   { gmm::write(o,m); return o; }
 
 
-#ifdef USING_BROKEN_GCC295
-  template <class M>
-  struct linalg_traits<const scaled_col_matrix_const_ref<M> > 
-    : public linalg_traits<scaled_col_matrix_const_ref<M> > {};
-#endif
-
-  template <class L, class R> struct __scaled_return {
+  template <typename L, typename R> struct scaled_return__ {
     typedef abstract_null_type return_type;
   };
-  template <class L> struct __scaled_return<L, row_major> 
+  template <typename L> struct scaled_return__<L, row_major> 
   { typedef scaled_row_matrix_const_ref<L> return_type; };
-  template <class L> struct __scaled_return<L, col_major> 
+  template <typename L> struct scaled_return__<L, col_major> 
   { typedef scaled_col_matrix_const_ref<L> return_type; };
   
 
-  template <class L, class LT> struct _scaled_return {
+  template <typename L, typename LT> struct scaled_return_ {
     typedef abstract_null_type return_type;
   };
-  template <class L> struct _scaled_return<L, abstract_vector> 
+  template <typename L> struct scaled_return_<L, abstract_vector> 
   { typedef scaled_vector_const_ref<L> return_type; };
-  template <class L> struct _scaled_return<L, abstract_matrix> {
-    typedef typename __scaled_return<L, 
+  template <typename L> struct scaled_return_<L, abstract_matrix> {
+    typedef typename scaled_return__<L, 
       typename principal_orientation_type<typename
       linalg_traits<L>::sub_orientation>::potype>::return_type return_type;
   };
 
-  template <class L> struct scaled_return {
-    typedef typename _scaled_return<L, typename
+  template <typename L> struct scaled_return {
+    typedef typename scaled_return_<L, typename
       linalg_traits<L>::linalg_type>::return_type return_type;
   };
 
-  template <class L> inline
+  template <typename L> inline
   typename scaled_return<L>::return_type
   scaled(const L &v, typename linalg_traits<L>::value_type x)
   { return scaled(v, x, typename linalg_traits<L>::linalg_type()); }
 
-  template <class V> inline
+  template <typename V> inline
   typename scaled_return<V>::return_type
   scaled(const V &v, typename linalg_traits<V>::value_type x, abstract_vector)
   { return scaled_vector_const_ref<V>(v, x); }
 
-  template <class M> inline
+  template <typename M> inline
   typename scaled_return<M>::return_type
   scaled(const M &m, typename linalg_traits<M>::value_type x,abstract_matrix) {
     return scaled(m, x,  typename principal_orientation_type<typename
 		  linalg_traits<M>::sub_orientation>::potype());
   }
 
-  template <class M> inline
+  template <typename M> inline
   typename scaled_return<M>::return_type
   scaled(const M &m, typename linalg_traits<M>::value_type x, row_major) {
     return scaled_row_matrix_const_ref<M>(m, x);
   }
 
-  template <class M> inline
+  template <typename M> inline
   typename scaled_return<M>::return_type
   scaled(const M &m, typename linalg_traits<M>::value_type x, col_major) {
     return scaled_col_matrix_const_ref<M>(m, x);
@@ -425,40 +385,40 @@ namespace gmm {
   /*	matrix or vector scale                                	          */
   /* ******************************************************************** */
 
-  template <class L> inline
+  template <typename L> inline
   void scale(L& l, typename linalg_traits<L>::value_type a)
   { scale(l, a, typename linalg_traits<L>::linalg_type()); }
 
-  template <class L> inline
+  template <typename L> inline
   void scale(const L& l, typename linalg_traits<L>::value_type a)
   { scale(linalg_const_cast(l), a); }
 
-  template <class L> inline
+  template <typename L> inline
   void scale(L& l, typename linalg_traits<L>::value_type a, abstract_vector) {
     typename linalg_traits<L>::iterator it = vect_begin(l), ite = vect_end(l);
     for ( ; it != ite; ++it) *it *= a;
   }
 
-  template <class L> 
+  template <typename L> 
   void scale(L& l, typename linalg_traits<L>::value_type a, abstract_matrix) {
     scale(l, a, typename principal_orientation_type<typename
 	  linalg_traits<L>::sub_orientation>::potype());
   }
 
-  template <class L> 
+  template <typename L> 
   void scale(L& l, typename linalg_traits<L>::value_type a, row_major) {
     typename linalg_traits<L>::row_iterator it = mat_row_begin(l),
       ite = mat_row_end(l);
-    for ( ; it != ite; ++it) scale(linalg_traits<L>::row(*it), a);
+    for ( ; it != ite; ++it) scale(linalg_traits<L>::row(it), a);
   }
 
-  template <class L> 
+  template <typename L> 
   void scale(L& l, typename linalg_traits<L>::value_type a, col_major) {
     typename linalg_traits<L>::col_iterator it = mat_col_begin(l),
       ite = mat_col_end(l);
-    for ( ; it != ite; ++it) scale(linalg_traits<L>::col(*it), a);
+    for ( ; it != ite; ++it) scale(linalg_traits<L>::col(it), a);
   }
 
 }
 
-#endif //  __GMM_SCALED_H
+#endif //  GMM_SCALED_H__
