@@ -11,7 +11,7 @@
 /*                                                                         */
 /* Copyright (C) 2002-2003 Yves Renard.                                    */
 /*                                                                         */
-/* This file is a part of GETFEM++                                         */
+/* This file is a part of GMM++                                            */
 /*                                                                         */
 /* This program is free software; you can redistribute it and/or modify    */
 /* it under the terms of the GNU Lesser General Public License as          */
@@ -29,10 +29,11 @@
 /*                                                                         */
 /* *********************************************************************** */
 
-#ifndef __GMM_VECTOR_H
-#define __GMM_VECTOR_H
+#ifndef GMM_VECTOR_H__
+#define GMM_VECTOR_H__
 
 #include <map>
+#include <gmm_interface.h>
 
 namespace gmm
 {
@@ -44,7 +45,7 @@ namespace gmm
   /*************************************************************************/
 
 
-  template<class T, class V> class ref_elt_vector {
+  template<typename T, typename V> class ref_elt_vector {
 
     V *pm;
     size_type l;
@@ -53,44 +54,85 @@ namespace gmm
 
     operator T() const { return pm->r(l); }
     ref_elt_vector(V *p, size_type ll) : pm(p), l(ll) {}
-    inline ref_elt_vector operator =(T v)
+    inline ref_elt_vector &operator =(T v)
       { (*pm).w(l,v); return *this; }
-    inline bool operator ==(T v) const
-      { return ((*pm).r(l) == v); }
-    inline bool operator !=(T v) const
-      { return ((*pm).r(l) != v); }
-    inline ref_elt_vector operator +=(T v)
+    inline bool operator ==(T v) const { return ((*pm).r(l) == v); }
+    inline bool operator !=(T v) const { return ((*pm).r(l) != v); }
+    inline ref_elt_vector &operator +=(T v)
       { (*pm).w(l,(*pm).r(l) + v); return *this; }
-    inline ref_elt_vector operator -=(T v)
+    inline ref_elt_vector &operator -=(T v)
       { (*pm).w(l,(*pm).r(l) - v); return *this; }
-    inline ref_elt_vector operator /=(T v)
+    inline ref_elt_vector &operator /=(T v)
       { (*pm).w(l,(*pm).r(l) / v); return *this; }
-    inline ref_elt_vector operator *=(T v)
+    inline ref_elt_vector &operator *=(T v)
       { (*pm).w(l,(*pm).r(l) * v); return *this; }
-    inline ref_elt_vector operator =(const ref_elt_vector &re)
+    inline ref_elt_vector &operator =(const ref_elt_vector &re)
       { *this = T(re); return *this; }
+    T operator +()    { return  T(*this);   } // necessary for unknow reason
+    T operator -()    { return -T(*this);   } // necessary for unknow reason
+    T operator +(T v) { return T(*this)+ v; } // necessary for unknow reason
+    T operator -(T v) { return T(*this)- v; } // necessary for unknow reason
+    T operator *(T v) { return T(*this)* v; } // necessary for unknow reason
+    T operator /(T v) { return T(*this)/ v; } // necessary for unknow reason
   };  
   
-  template<class T, class V> T operator +(const ref_elt_vector<T, V> &re)
-    { return T(re); }
-  template<class T, class V> T operator -(const ref_elt_vector<T, V> &re)
-    { return -T(re); }
-  template<class T, class V> T operator +(const ref_elt_vector<T, V> &re, T v)
-    { return T(re)+ v; }
-  template<class T, class V> T operator +(T v, const ref_elt_vector<T, V> &re)
-    { return v+ T(re); }
-  template<class T, class V> T operator -(const ref_elt_vector<T, V> &re, T v)
-    { return T(re)- v; }
-  template<class T, class V> T operator -(T v, const ref_elt_vector<T, V> &re)
-    { return v- T(re); }
-  template<class T, class V> T operator *(const ref_elt_vector<T, V> &re, T v)
-    { return T(re)* v; }
-  template<class T, class V> T operator *(T v, const ref_elt_vector<T, V> &re)
-    { return v* T(re); }
-  template<class T, class V> T operator /(const ref_elt_vector<T, V> &re, T v)
-    { return T(re)/ v; }
-  template<class T, class V> T operator /(T v, const ref_elt_vector<T, V> &re)
-    { return v/ T(re); }
+  
+  template<typename T, typename V> inline
+  bool operator ==(T v, const ref_elt_vector<T, V> &re) { return (v==T(re)); }
+  template<typename T, typename V> inline
+  bool operator !=(T v, const ref_elt_vector<T, V> &re) { return (v!=T(re)); }
+  template<typename T, typename V> inline
+  T &operator +=(T &v, const ref_elt_vector<T, V> &re) 
+  { v += T(re); return v; }
+  template<typename T, typename V> inline
+  T &operator -=(T &v, const ref_elt_vector<T, V> &re)
+  { v -= T(re); return v; }
+  template<typename T, typename V> inline
+  T &operator *=(T &v, const ref_elt_vector<T, V> &re) 
+  { v *= T(re); return v; }
+  template<typename T, typename V> inline
+  T &operator /=(T &v, const ref_elt_vector<T, V> &re)
+  { v /= T(re); return v; }
+  template<typename T, typename V> inline
+  T operator +(const ref_elt_vector<T, V> &re) { return T(re); }
+  template<typename T, typename V> inline
+  T operator -(const ref_elt_vector<T, V> &re) { return -T(re); }
+  template<typename T, typename V> inline
+  T operator +(const ref_elt_vector<T, V> &re, T v) { return T(re)+ v; }
+  template<typename T, typename V> inline
+  T operator +(T v, const ref_elt_vector<T, V> &re) { return v+ T(re); }
+  template<typename T, typename V> inline
+  T operator -(const ref_elt_vector<T, V> &re, T v) { return T(re)- v; }
+  template<typename T, typename V> inline
+  T operator -(T v, const ref_elt_vector<T, V> &re) { return v- T(re); }
+  template<typename T, typename V>  inline
+  T operator *(const ref_elt_vector<T, V> &re, T v) { return T(re)* v; }
+  template<typename T, typename V> inline
+  T operator *(T v, const ref_elt_vector<T, V> &re) { return v* T(re); }
+  template<typename T, typename V> inline
+  T operator /(const ref_elt_vector<T, V> &re, T v) { return T(re)/ v; }
+  template<typename T, typename V> inline
+  T operator /(T v, const ref_elt_vector<T, V> &re) { return v/ T(re); }
+  template<typename T, typename V> inline
+  typename number_traits<T>::magnitude_type
+  abs(const ref_elt_vector<T, V> &re) { return gmm::abs(T(re)); }
+  template<typename T, typename V> inline
+  T sqr(const ref_elt_vector<T, V> &re) { return gmm::sqr(T(re)); }
+  template<typename T, typename V> inline
+  typename number_traits<T>::magnitude_type
+  abs_sqr(const ref_elt_vector<T, V> &re) { return gmm::abs_sqr(T(re)); }
+  template<typename T, typename V> inline
+  T conj(const ref_elt_vector<T, V> &re) { return gmm::conj(T(re)); }
+  template<typename T, typename V> std::ostream &operator <<
+  (std::ostream &o, const ref_elt_vector<T, V> &re) { o << T(re); return o; }
+  template<typename T, typename V> inline
+  typename number_traits<T>::magnitude_type
+  real(const ref_elt_vector<T, V> &re) { return gmm::real(T(re)); }
+  template<typename T, typename V> inline
+  typename number_traits<T>::magnitude_type
+  imag(const ref_elt_vector<T, V> &re) { return gmm::imag(T(re)); }
+
+  
 
   /*************************************************************************/
   /*                                                                       */
@@ -98,7 +140,7 @@ namespace gmm
   /*                                                                       */
   /*************************************************************************/
   
-  template<class T> struct wsvector_iterator
+  template<typename T> struct wsvector_iterator
     : public std::map<size_type, T>::iterator {
     typedef typename std::map<size_type, T>::iterator base_it_type;
     typedef T                   value_type;
@@ -116,7 +158,7 @@ namespace gmm
     wsvector_iterator(const base_it_type &it) : base_it_type(it) {}
   };
 
-  template<class T> struct wsvector_const_iterator
+  template<typename T> struct wsvector_const_iterator
     : public std::map<size_type, T>::const_iterator {
     typedef typename std::map<size_type, T>::const_iterator base_it_type;
     typedef T                   value_type;
@@ -137,7 +179,7 @@ namespace gmm
   };
 
 
-  template<class T> class wsvector : public std::map<size_type, T> {
+  template<typename T> class wsvector : public std::map<size_type, T> {
   public:
     
     typedef typename std::map<int, T>::size_type size_type;
@@ -151,21 +193,21 @@ namespace gmm
   public:
     void out_of_range_error(void) const;
     void clean(double eps);
-    void resize(size_type l) { nbl = l;  /* + suprimer les elements en trop */}
+    void resize(size_type);
     
     inline ref_elt_vector<T, wsvector<T> > operator [](size_type c)
     { return ref_elt_vector<T, wsvector<T> >(this, c); }
 
     inline void w(size_type c, const T &e) {
-#   ifdef __GETFEM_VERIFY
+#   ifdef GMM_VERIFY
       if (c >= nbl) out_of_range_error();
 #   endif
-      if (e == T(0)) erase(c);
+      if (e == T(0)) { base_type::erase(c); }
       else base_type::operator [](c) = e;
     }
 
     inline T r(size_type c) const {
-#ifdef __GETFEM_VERIFY
+#ifdef GMM_VERIFY
       if (c >= nbl) out_of_range_error();
 #endif
       const_iterator it = lower_bound(c);
@@ -178,48 +220,37 @@ namespace gmm
     size_type nb_stored(void) const { return base_type::size(); }
     size_type size(void) const { return nbl; }
 
+    void swap(wsvector<T> &v)
+    { std::swap(nbl, v.nbl); std::map<size_type, T>::swap(v); }
+				       
+
     /* Constructeurs */
     void init(size_type l) { nbl = l; this->clear(); }
     explicit wsvector(size_type l){ init(l); }
     wsvector(void) { init(0); }
   };
 
-  template<class T>  void wsvector<T>::clean(double eps) {
-    iterator it = this->begin(), itf = ++it, ite = this->end();
-    for ( ; it != ite; ++itf)
-      { if (dal::abs((*it).e) <= eps) { erase(it); it = itf; } else ++it; }
+  template<typename T>  void wsvector<T>::clean(double eps) {
+    iterator it = this->begin(), itf = it, ite = this->end();
+    while (it != ite) {
+      ++itf; if (gmm::abs(it->second) <= eps) erase(it); it = itf;
+    }
   }
 
-  template<class T>  void wsvector<T>::out_of_range_error(void) const
+  template<typename T>  void wsvector<T>::resize(size_type n) {
+    if (n < nbl) {
+      iterator it = this->begin(), itf = it, ite = this->end();
+      while (it != ite) { ++itf; if (it->first >= n) erase(it); it = itf; }
+    }
+    nbl = n;
+  }
+
+  template<typename T>  void wsvector<T>::out_of_range_error(void) const
   { DAL_THROW(std::out_of_range, "out of range"); }
 
-  template <class T> struct wsvector_access {
-    typedef wsvector<T> V;
-    typedef typename linalg_traits<V>::value_type value_type;
-    typedef typename linalg_traits<V>::reference reference;
-    typedef typename linalg_traits<V>::iterator iterator;
-    typedef typename linalg_traits<V>::const_iterator const_iterator;
-    
-    reference operator()(const void *o, const iterator &,
-			 const iterator &, size_type i)
-    { return (*(const_cast<V *>((const V *)(o))))[i]; }
-
-    value_type operator()(const void *o, const const_iterator &,
-			 const const_iterator &, size_type i)
-    { return (*(const V *)(o))[i]; }
-
-  };
-
-  template <class T> struct wsvector_clear {
-    typedef wsvector<T> V;
-    typedef typename linalg_traits<V>::iterator iterator;
-    
-    void operator()(const void *o, const iterator &, const iterator &)
-    { (const_cast<V *>((const V *)(o)))->clear(); }
-  };
-
-  template <class T> struct linalg_traits<wsvector<T> > {
+  template <typename T> struct linalg_traits<wsvector<T> > {
     typedef wsvector<T> this_type;
+    typedef this_type origin_type;
     typedef linalg_false is_reference;
     typedef abstract_vector linalg_type;
     typedef T value_type;
@@ -227,69 +258,72 @@ namespace gmm
     typedef wsvector_iterator<T>  iterator;
     typedef wsvector_const_iterator<T> const_iterator;
     typedef abstract_sparse storage_type;
-    typedef wsvector_access<T> access_type;
-    typedef wsvector_clear<T> clear_type;
     static size_type size(const this_type &v) { return v.size(); }
     static iterator begin(this_type &v) { return v.begin(); }
     static const_iterator begin(const this_type &v) { return v.begin(); }
     static iterator end(this_type &v) { return v.end(); }
     static const_iterator end(const this_type &v) { return v.end(); }
-    static const void* origin(const this_type &v) { return &v; }
-    static void do_clear(this_type &v)
-    { clear_type()(origin(v), begin(v), end(v)); }
+    static origin_type* origin(this_type &v) { return &v; }
+    static const origin_type* origin(const this_type &v) { return &v; }
+    static void clear(origin_type* o, const iterator &, const iterator &)
+    { o->clear(); }
+    static void do_clear(this_type &v) { v.clear(); }
+    static value_type access(const origin_type *o, const const_iterator &,
+			     const const_iterator &, size_type i)
+    { return (*o)[i]; }
+    static reference access(origin_type *o, const iterator &, const iterator &,
+			    size_type i)
+    { return (*o)[i]; }
+    static void resize(this_type &v, size_type n) { v.resize(n); }
   };
 
-  template<class T> std::ostream &operator <<
+  template<typename T> std::ostream &operator <<
   (std::ostream &o, const wsvector<T>& v) { gmm::write(o,v); return o; }
 
-#ifdef USING_BROKEN_GCC295
-  template <class T> struct linalg_traits<const wsvector<T> >
-    : public linalg_traits<wsvector<T> > {};
-#endif
-
   /******* Optimized BLAS for wsvector<T> **********************************/
 
-  template <class T> inline void copy(const wsvector<T> &v1, wsvector<T> &v2) {
+  template <typename T> inline void copy(const wsvector<T> &v1, wsvector<T> &v2) {
     if (vect_size(v1) != vect_size(v2))
       DAL_THROW(dimension_error,"dimensions mismatch");
     v2 = v1;
   }
-  template <class T> inline
+  template <typename T> inline
   void copy(const wsvector<T> &v1, const simple_vector_ref<wsvector<T> *> &v2){
     simple_vector_ref<wsvector<T> *>
       *svr = const_cast<simple_vector_ref<wsvector<T> *> *>(&v2);
     wsvector<T>
-      *pv = const_cast<wsvector<T> *>((const wsvector<T> *)(v2.origin));
+      *pv = const_cast<wsvector<T> *>(v2.origin);
     if (vect_size(v1) != vect_size(v2))
 	DAL_THROW(dimension_error,"dimensions mismatch");
-    *pv = v1; svr->_begin = vect_begin(*pv); svr->_end = vect_end(*pv);
+    *pv = v1; svr->begin_ = vect_begin(*pv); svr->end_ = vect_end(*pv);
   }
-  template <class T> inline
+  template <typename T> inline
   void copy(const simple_vector_ref<const wsvector<T> *> &v1,
 	    wsvector<T> &v2)
-  { copy(*(const wsvector<T> *)(v1.origin), v2); }
-  template <class T> inline
+  { copy(*(v1.origin), v2); }
+  template <typename T> inline
   void copy(const simple_vector_ref<wsvector<T> *> &v1, wsvector<T> &v2)
-  { copy(*(const wsvector<T> *)(v1.origin), v2); }
+  { copy(*(v1.origin), v2); }
 
-  template <class T> inline void clean(wsvector<T> &v, double eps) {
+  template <typename T> inline void clean(wsvector<T> &v, double eps) {
+    typedef typename number_traits<T>::magnitude_type R;
     typename wsvector<T>::iterator it = v.begin(), ite = v.end(), itc;
     while (it != ite) 
-      if (dal::abs((*it).second) <= eps)
+      if (gmm::abs((*it).second) <= R(eps))
 	{ itc=it; ++it; v.erase(itc); } else ++it; 
   }
 
-  template <class T>
+  template <typename T>
   inline void clean(const simple_vector_ref<wsvector<T> *> &l, double eps) {
     simple_vector_ref<wsvector<T> *>
       *svr = const_cast<simple_vector_ref<wsvector<T> *> *>(&l);
     wsvector<T>
-      *pv = const_cast<wsvector<T> *>((const wsvector<T> *)(l.origin));
+      *pv = const_cast<wsvector<T> *>((l.origin));
     clean(*pv, eps);
-    svr->_begin = vect_begin(*pv); svr->_end = vect_end(*pv);
+    svr->begin_ = vect_begin(*pv); svr->end_ = vect_end(*pv);
   }
 
-  template <class T>
+  template <typename T>
   inline size_type nnz(const wsvector<T>& l) { return l.nb_stored(); }
 
   /*************************************************************************/
@@ -298,18 +332,18 @@ namespace gmm
   /*                                                                       */
   /*************************************************************************/
 
-  template<class T> struct _elt_rsvector {
+  template<typename T> struct elt_rsvector_ {
     size_type c; T e;
-    _elt_rsvector(void) {  }
-    _elt_rsvector(size_type cc) { c = cc; }
-    _elt_rsvector(size_type cc, const T &ee) { c = cc; e = ee; }
-    bool operator < (const _elt_rsvector &a) const { return c < a.c; }
-    bool operator == (const _elt_rsvector &a) const { return c == a.c; }
-    bool operator != (const _elt_rsvector &a) const { return c != a.c; }
+    elt_rsvector_(void) {  }
+    elt_rsvector_(size_type cc) { c = cc; }
+    elt_rsvector_(size_type cc, const T &ee) { c = cc; e = ee; }
+    bool operator < (const elt_rsvector_ &a) const { return c < a.c; }
+    bool operator == (const elt_rsvector_ &a) const { return c == a.c; }
+    bool operator != (const elt_rsvector_ &a) const { return c != a.c; }
   };
 
-  template<class T> struct rsvector_iterator {
-    typedef typename std::vector<_elt_rsvector<T> >::iterator IT;
+  template<typename T> struct rsvector_iterator {
+    typedef typename std::vector<elt_rsvector_<T> >::iterator IT;
     typedef T                   value_type;
     typedef value_type*         pointer;
     typedef value_type&         reference;
@@ -336,8 +370,8 @@ namespace gmm
     rsvector_iterator(const IT &i) : it(i) {}
   };
 
-  template<class T> struct rsvector_const_iterator {
-    typedef typename std::vector<_elt_rsvector<T> >::const_iterator IT;
+  template<typename T> struct rsvector_const_iterator {
+    typedef typename std::vector<elt_rsvector_<T> >::const_iterator IT;
     typedef T                   value_type;
     typedef const value_type*   pointer;
     typedef const value_type&   reference;
@@ -365,24 +399,24 @@ namespace gmm
     rsvector_const_iterator(const IT &i) : it(i) {}
   };
 
-  template<class T> class rsvector : public std::vector<_elt_rsvector<T> > {
+  template<typename T> class rsvector : public std::vector<elt_rsvector_<T> > {
   public:
     
-    typedef std::vector<_elt_rsvector<T> > _base_type;
-    typedef typename _base_type::iterator iterator;
-    typedef typename _base_type::const_iterator const_iterator;
-    typedef typename _base_type::size_type size_type;
+    typedef std::vector<elt_rsvector_<T> > base_type_;
+    typedef typename base_type_::iterator iterator;
+    typedef typename base_type_::const_iterator const_iterator;
+    typedef typename base_type_::size_type size_type;
     typedef T value_type;
 
   protected:
-    size_type nbl;    	/* Nombre d'elements max.	        	  */
+    size_type nbl;    	/* size of the vector.	          	  */
     
   public:
 
     void sup(size_type j);
     void out_of_range_error(void) const;
-    void base_resize(size_type n) { _base_type::resize(n); }
-    void resize(size_type l) { nbl = l; /* + suprimer les elements en trop*/ }
+    void base_resize(size_type n) { base_type_::resize(n); }
+    void resize(size_type);
     
     ref_elt_vector<T, rsvector<T> > operator [](size_type c)
     { return ref_elt_vector<T, rsvector<T> >(this, c); }
@@ -392,34 +426,45 @@ namespace gmm
 
     inline T operator [](size_type c) const { return r(c); }
     
-    size_type nb_stored(void) const { return _base_type::size(); }
+    size_type nb_stored(void) const { return base_type_::size(); }
     size_type size(void) const { return nbl; }
-    void clear(void) { _base_type::resize(0); }
+    void clear(void) { base_type_::resize(0); }
+    void swap(rsvector<T> &v)
+    { std::swap(nbl, v.nbl); std::vector<elt_rsvector_<T> >::swap(v); }
+
     /* Constructeurs */
     explicit rsvector(size_type l) : nbl(l) { }
     rsvector(void) : nbl(0) { }
   };
 
-  template <class T> void rsvector<T>::sup(size_type j) {
+  template <typename T> void rsvector<T>::sup(size_type j) {
     if (nb_stored() != 0) {
-      _elt_rsvector<T> ev(j);
+      elt_rsvector_<T> ev(j);
       iterator it = std::lower_bound(this->begin(), this->end(), ev);
       if (it != this->end() && it->c == j) {
 	for (iterator ite = this->end() - 1; it != ite; ++it) *it = *(it+1);
-	_base_type::resize(nb_stored()-1);
+	base_type_::resize(nb_stored()-1);
       }
     }
   }
 
-  template <class T> void rsvector<T>::w(size_type c, const T &e) {
-#   ifdef __GETFEM_VERIFY
+  template<typename T>  void rsvector<T>::resize(size_type n) {
+    if (n < nbl) {
+      for (size_type i = 0; i < nb_stored(); ++i)
+	if (base_type_::operator[](i).c >= n) { base_resize(i); break; }
+    }
+    nbl = n;
+  }
+
+  template <typename T> void rsvector<T>::w(size_type c, const T &e) {
+#   ifdef GMM_VERIFY
     if (c >= nbl) out_of_range_error();
 #   endif
     if (e == T(0)) sup(c);
     else {
-      _elt_rsvector<T> ev(c, e);
+      elt_rsvector_<T> ev(c, e);
       if (nb_stored() == 0) {
-	_base_type::resize(1);
+	base_type_::resize(1);
 	*(this->begin()) = ev;
       }
       else {
@@ -427,7 +472,7 @@ namespace gmm
 	if (it != this->end() && it->c == c) it->e = e;
 	else {
 	  size_type ind = it - this->begin();
-	  _base_type::resize(nb_stored()+1);
+	  base_type_::resize(nb_stored()+1);
 	  it = this->begin() + ind;
 	  for (iterator ite = this->end() - 1; ite != it; --ite)
 	    *ite = *(ite-1);
@@ -437,48 +482,24 @@ namespace gmm
     }
   }
   
-  template <class T> T rsvector<T>::r(size_type c) const {
-#   ifdef __GETFEM_VERIFY
+  template <typename T> T rsvector<T>::r(size_type c) const {
+#   ifdef GMM_VERIFY
     if (c >= nbl) out_of_range_error();
 #   endif
     if (nb_stored() != 0) {
-      _elt_rsvector<T> ev(c);
+      elt_rsvector_<T> ev(c);
       const_iterator it = std::lower_bound(this->begin(), this->end(), ev);
       if (it != this->end() && it->c == c) return it->e;
     }
     return T(0);
   }
 
-  template<class T>  void rsvector<T>::out_of_range_error(void) const
+  template<typename T>  void rsvector<T>::out_of_range_error(void) const
   { DAL_THROW(std::out_of_range, "out of range"); }
 
-  template <class T> struct rsvector_access {
-    typedef rsvector<T> V;
-    typedef typename linalg_traits<V>::value_type value_type;
-    typedef typename linalg_traits<V>::reference reference;
-    typedef typename linalg_traits<V>::iterator iterator;
-    typedef typename linalg_traits<V>::const_iterator const_iterator;
-    
-    reference operator()(const void *o, const iterator &,
-			 const iterator &, size_type i)
-    { return (*(const_cast<V *>((const V *)(o))))[i]; }
-
-    value_type operator()(const void *o, const const_iterator &,
-			 const const_iterator &, size_type i)
-    { return (*(const V *)(o))[i]; }
-
-  };
-
-  template <class T> struct rsvector_clear {
-    typedef rsvector<T> V;
-    typedef typename linalg_traits<V>::iterator iterator;
-    
-    void operator()(const void *o, const iterator &, const iterator &)
-    { (*(const_cast<V *>((const V *)(o)))).clear(); }
-  };
-
-  template <class T> struct linalg_traits<rsvector<T> > {
+  template <typename T> struct linalg_traits<rsvector<T> > {
     typedef rsvector<T> this_type;
+    typedef this_type origin_type;
     typedef linalg_false is_reference;
     typedef abstract_vector linalg_type;
     typedef T value_type;
@@ -486,8 +507,6 @@ namespace gmm
     typedef rsvector_iterator<T>  iterator;
     typedef rsvector_const_iterator<T> const_iterator;
     typedef abstract_sparse storage_type;
-    typedef rsvector_access<T> access_type;
-    typedef rsvector_clear<T> clear_type;
     static size_type size(const this_type &v) { return v.size(); }
     static iterator begin(this_type &v) { return iterator(v.begin()); }
     static const_iterator begin(const this_type &v)
@@ -495,45 +514,49 @@ namespace gmm
     static iterator end(this_type &v) { return iterator(v.end()); }
     static const_iterator end(const this_type &v)
       { return const_iterator(v.end()); }
-    static const void* origin(const this_type &v) { return &v; }
-    static void do_clear(this_type &v)
-      { clear_type()(origin(v), begin(v), end(v)); }
+    static origin_type* origin(this_type &v) { return &v; }
+    static const origin_type* origin(const this_type &v) { return &v; }
+    static void clear(origin_type* o, const iterator &, const iterator &)
+    { o->clear(); }
+    static void do_clear(this_type &v) { v.clear(); }
+    static value_type access(const origin_type *o, const const_iterator &,
+			     const const_iterator &, size_type i)
+    { return (*o)[i]; }
+    static reference access(origin_type *o, const iterator &, const iterator &,
+			    size_type i)
+    { return (*o)[i]; }
+    static void resize(this_type &v, size_type n) { v.resize(n); }
   };
 
-  template<class T> std::ostream &operator <<
+  template<typename T> std::ostream &operator <<
   (std::ostream &o, const rsvector<T>& v) { gmm::write(o,v); return o; }
 
-#ifdef USING_BROKEN_GCC295
-  template <class T> struct linalg_traits<const rsvector<T> >
-    : public linalg_traits<rsvector<T> > {};
-#endif
+  /******* Optimized operations for rsvector<T> ****************************/
 
-  /******* Optimized BLAS for rsvector<T> **********************************/
-
-  template <class T> inline void copy(const rsvector<T> &v1, rsvector<T> &v2) {
+  template <typename T> inline void copy(const rsvector<T> &v1, rsvector<T> &v2) {
     if (vect_size(v1) != vect_size(v2))
       DAL_THROW(dimension_error,"dimensions mismatch");
     v2 = v1;
   }
-  template <class T> inline
+  template <typename T> inline
   void copy(const rsvector<T> &v1, const simple_vector_ref<rsvector<T> *> &v2){
     simple_vector_ref<rsvector<T> *>
       *svr = const_cast<simple_vector_ref<rsvector<T> *> *>(&v2);
     rsvector<T>
-      *pv = const_cast<rsvector<T> *>((const rsvector<T> *)(v2.origin));
+      *pv = const_cast<rsvector<T> *>((v2.origin));
     if (vect_size(v1) != vect_size(v2))
 	DAL_THROW(dimension_error,"dimensions mismatch");
-    *pv = v1; svr->_begin = vect_begin(*pv); svr->_end = vect_end(*pv);
+    *pv = v1; svr->begin_ = vect_begin(*pv); svr->end_ = vect_end(*pv);
   }
-  template <class T> inline
+  template <typename T> inline
   void copy(const simple_vector_ref<const rsvector<T> *> &v1,
 	    rsvector<T> &v2)
-  { copy(*(const rsvector<T> *)(v1.origin), v2); }
-  template <class T> inline
+  { copy(*(v1.origin), v2); }
+  template <typename T> inline
   void copy(const simple_vector_ref<rsvector<T> *> &v1, rsvector<T> &v2)
-  { copy(*(const rsvector<T> *)(v1.origin), v2); }
+  { copy(*(v1.origin), v2); }
 
-  template <class V, class T> inline void add(const V &v1, rsvector<T> &v2) {
+  template <typename V, typename T> inline void add(const V &v1, rsvector<T> &v2) {
     if ((const void *)(&v1) != (const void *)(&v2)) {
       if (vect_size(v1) != vect_size(v2))
 	DAL_THROW(dimension_error,"dimensions mismatch");
@@ -541,15 +564,15 @@ namespace gmm
     }
   }
 
-  template <class V, class T> 
+  template <typename V, typename T> 
   inline void add_rsvector(const V &v1, rsvector<T> &v2, abstract_dense)
   { add(v1, v2, abstract_dense(), abstract_sparse()); }
 
-  template <class V, class T> 
+  template <typename V, typename T> 
   inline void add_rsvector(const V &v1, rsvector<T> &v2, abstract_skyline)
   { add(v1, v2, abstract_skyline(), abstract_sparse()); }
 
-  template <class V, class T> 
+  template <typename V, typename T> 
   void add_rsvector(const V &v1, rsvector<T> &v2, abstract_sparse) {
     typename linalg_traits<V>::const_iterator it1 = vect_const_begin(v1),
       ite1 = vect_const_end(v1);
@@ -562,39 +585,39 @@ namespace gmm
     for (; it2 != ite2; ++it2) ++nbc;
 
     v2.base_resize(nbc);
-    it3 = v2.begin() + old_nbc; --it3;
-    it2 = v2.end(); --it2; ite2 = v2.begin(); --ite2;
-    it1 = vect_end(v1); --it1; ite1 = vect_const_begin(v1); --ite1;
-
-    for (; it1 != ite1 && it3 != ite2; --it2) {
-      if (it3->c > it1.index()) { *it2 = *it3; --it3; }
-      else if (it3->c == it1.index()) { *it2=*it3; it2->e+=*it1; --it3; --it1;}
-      else { it2->c = it1.index(); it2->e = *it1; --it1; }
+    it3 = v2.begin() + old_nbc;
+    it2 = v2.end(); ite2 = v2.begin();
+    it1 = vect_end(v1); ite1 = vect_const_begin(v1);
+    while (it1 != ite1 && it3 != ite2) {
+      --it3; --it1; --it2;
+      if (it3->c > it1.index()) { *it2 = *it3; ++it1; }
+      else if (it3->c == it1.index()) { *it2=*it3; it2->e+=*it1; }
+      else { it2->c = it1.index(); it2->e = *it1; ++it3; }
     }
-    for (; it1 != ite1; --it2) { it2->c = it1.index(); it2->e = *it1; --it1; }
+    while (it1 != ite1) { --it1; --it2; it2->c = it1.index(); it2->e = *it1; }
   }
 
-  template <class V, class T> void copy(const V &v1, rsvector<T> &v2) {
+  template <typename V, typename T> void copy(const V &v1, rsvector<T> &v2) {
     if ((const void *)(&v1) != (const void *)(&v2)) {
       if (vect_size(v1) != vect_size(v2))
 	DAL_THROW(dimension_error,"dimensions mismatch");
-#       ifdef __GETFEM_VERIFY
-        if (linalg_origin(v1) == linalg_origin(v2))
+#       ifdef GMM_VERIFY
+         if (same_origin(v1, v2))
 	  DAL_WARNING(2, "a conflict is possible in vector copy\n");
 #       endif
 	copy_rsvector(v1, v2, typename linalg_traits<V>::storage_type());
     }
   }
 
-  template <class V, class T> 
+  template <typename V, typename T> 
   void copy_rsvector(const V &v1, rsvector<T> &v2, abstract_dense)
   { copy_vect(v1, v2, abstract_dense(), abstract_sparse()); }
 
-  template <class V, class T> 
+  template <typename V, typename T> 
   void copy_rsvector(const V &v1, rsvector<T> &v2, abstract_skyline)
   { copy_vect(v1, v2, abstract_skyline(), abstract_sparse()); }
 
-  template <class V, class T> //  refaire
+  template <typename V, typename T> //  refaire
   void copy_rsvector(const V &v1, rsvector<T> &v2, abstract_sparse) {
      typename linalg_traits<V>::const_iterator it = vect_const_begin(v1),
       ite = vect_const_end(v1);
@@ -613,29 +636,30 @@ namespace gmm
     }
   }
   
-  template <class T> inline void clean(rsvector<T> &v, double eps) {
+  template <typename T> inline void clean(rsvector<T> &v, double eps) {
+    typedef typename number_traits<T>::magnitude_type R;
     typename rsvector<T>::iterator it = v.begin(), ite = v.end();
-    for (; it != ite; ++it) if (dal::abs((*it).e) <= eps) break;
+    for (; it != ite; ++it) if (gmm::abs((*it).e) <= eps) break;
     if (it != ite) {
       typename rsvector<T>::iterator itc = it;
       size_type erased = 1;
       for (++it; it != ite; ++it)
-	{ *itc = *it; if (dal::abs((*it).e) <= eps) ++erased; else ++itc; }
+	{ *itc = *it; if (gmm::abs((*it).e) <= R(eps)) ++erased; else ++itc; }
       v.base_resize(v.nb_stored() - erased);
     }
   }
 
-  template <class T>
+  template <typename T>
   inline void clean(const simple_vector_ref<rsvector<T> *> &l, double eps) {
     simple_vector_ref<rsvector<T> *>
       *svr = const_cast<simple_vector_ref<rsvector<T> *> *>(&l);
     rsvector<T>
-      *pv = const_cast<rsvector<T> *>((const rsvector<T> *)(l.origin));
+      *pv = const_cast<rsvector<T> *>((l.origin));
     clean(*pv, eps);
-    svr->_begin = vect_begin(*pv); svr->_end = vect_end(*pv);
+    svr->begin_ = vect_begin(*pv); svr->end_ = vect_end(*pv);
   }
   
-  template <class T>
+  template <typename T>
   inline size_type nnz(const rsvector<T>& l) { return l.nb_stored(); }
 
   /*************************************************************************/
@@ -644,7 +668,7 @@ namespace gmm
   /*                                                                       */
   /*************************************************************************/
 
-  template<class T> struct slvector_iterator
+  template<typename T> struct slvector_iterator
   {
     typedef T value_type;
     typedef T *pointer;
@@ -696,7 +720,7 @@ namespace gmm
       : it(iter), shift(s) {}
   };
 
-  template<class T> struct slvector_const_iterator
+  template<typename T> struct slvector_const_iterator
   {
     typedef T value_type;
     typedef const T *pointer;
@@ -751,7 +775,7 @@ namespace gmm
   };
 
 
-  template <class T> class slvector {
+  template <typename T> class slvector {
     
   public :
     typedef slvector_iterator<T> iterators;
@@ -762,13 +786,13 @@ namespace gmm
   protected :
     std::vector<T> data;
     size_type shift;
-    size_type _size;
+    size_type size_;
 
 
   public :
 
     void out_of_range_error(void) const;
-    size_type size(void) const { return _size; }
+    size_type size(void) const { return size_; }
     size_type first(void) const { return shift; }
     size_type last(void) const { return shift + data.size(); }
     ref_elt_vector<T, slvector<T> > operator [](size_type c)
@@ -783,28 +807,40 @@ namespace gmm
 
     void w(size_type c, const T &e);
     T r(size_type c) const {
-#   ifdef __GETFEM_VERIFY
-      if (c >= _size) out_of_range_error();
+#   ifdef GMM_VERIFY
+      if (c >= size_) out_of_range_error();
 #   endif
       if (c < shift || c >= shift + data.size()) return T(0);
       return data[c - shift];
     }
 
     inline T operator [](size_type c) const { return r(c); }
-
+    void resize(size_type);
     void clear(void) { data.resize(0); shift = 0; }
+    void swap(slvector<T> &v) {
+      std::swap(data, v.data);
+      std::swap(shift, v.shift);
+      std::swap(size_, v.size_);
+    }
 
-    slvector(void) : data(0), shift(0), _size(0) {}
-    explicit slvector(size_type l) : data(0), shift(0), _size(l) {}
+
+    slvector(void) : data(0), shift(0), size_(0) {}
+    explicit slvector(size_type l) : data(0), shift(0), size_(l) {}
     slvector(size_type l, size_type d, size_type s)
-      : data(d), shift(d), _size(l) {}
+      : data(d), shift(d), size_(l) {}
 
   };
 
-  template<class T>  void slvector<T>::w(size_type c, const T &e) {
-    // cout << "vecteur avant : " << *this << " ajout  l'indice " << c << " de " << e << endl;
-#   ifdef __GETFEM_VERIFY
-      if (c >= _size) out_of_range_error();
+  template<typename T>  void slvector<T>::resize(size_type n) {
+    if (n < last()) {
+      if (shift >= n) clear(); else { data.resize(n-shift); }
+    }
+    size_ = n;
+  }
+
+  template<typename T>  void slvector<T>::w(size_type c, const T &e) {
+#   ifdef GMM_VERIFY
+      if (c >= size_) out_of_range_error();
 #   endif
       size_type s = data.size();
       if (!s) { data.resize(1); shift = c; }
@@ -821,39 +857,14 @@ namespace gmm
 	std::fill(data.begin() + s, data.end(), T(0));
       }
       data[c - shift] = e;
-      // cout << "rsultat : " << *this << endl;
     }
 
-  template<class T>  void slvector<T>::out_of_range_error(void) const
+  template<typename T>  void slvector<T>::out_of_range_error(void) const
   { DAL_THROW(std::out_of_range, "out of range"); }
 
-  template <class T> struct slvector_access {
-    typedef slvector<T> V;
-    typedef typename linalg_traits<V>::value_type value_type;
-    typedef typename linalg_traits<V>::reference reference;
-    typedef typename linalg_traits<V>::iterator iterator;
-    typedef typename linalg_traits<V>::const_iterator const_iterator;
-    
-    reference operator()(const void *o, const iterator &,
-			 const iterator &, size_type i)
-    { return (*(const_cast<V *>((const V *)(o))))[i]; }
-
-    value_type operator()(const void *o, const const_iterator &,
-			 const const_iterator &, size_type i)
-    { return (*(const V *)(o))[i]; }
-
-  };
-
-  template <class T> struct slvector_clear {
-    typedef slvector<T> V;
-    typedef typename linalg_traits<V>::iterator iterator;
-    
-    void operator()(const void *o, const iterator &, const iterator &)
-    { (*(const_cast<V *>((const V *)(o)))).clear(); }
-  };
-
-  template <class T> struct linalg_traits<slvector<T> > {
+  template <typename T> struct linalg_traits<slvector<T> > {
     typedef slvector<T> this_type;
+    typedef this_type origin_type;
     typedef linalg_false is_reference;
     typedef abstract_vector linalg_type;
     typedef T value_type;
@@ -861,8 +872,6 @@ namespace gmm
     typedef slvector_iterator<T>  iterator;
     typedef slvector_const_iterator<T> const_iterator;
     typedef abstract_skyline storage_type;
-    typedef slvector_access<T> access_type;
-    typedef slvector_clear<T> clear_type;
     static size_type size(const this_type &v) { return v.size(); }
     static iterator begin(this_type &v)
       { return iterator(v.data_begin(), v.first()); }
@@ -872,22 +881,57 @@ namespace gmm
       { return iterator(v.data_end(), v.last()); }
     static const_iterator end(const this_type &v)
       { return const_iterator(v.data_end(), v.last()); }
-    static const void* origin(const this_type &v) { return &v; }
-    static void do_clear(this_type &v)
-      { clear_type()(origin(v), begin(v), end(v)); }
+    static origin_type* origin(this_type &v) { return &v; }
+    static const origin_type* origin(const this_type &v) { return &v; }
+    static void clear(origin_type* o, const iterator &, const iterator &)
+    { o->clear(); }
+    static void do_clear(this_type &v) { v.clear(); }
+    static value_type access(const origin_type *o, const const_iterator &,
+			     const const_iterator &, size_type i)
+    { return (*o)[i]; }
+    static reference access(origin_type *o, const iterator &, const iterator &,
+			    size_type i)
+    { return (*o)[i]; }
+    static void resize(this_type &v, size_type n) { v.resize(n); }
   };
 
-  template<class T> std::ostream &operator <<
+  template<typename T> std::ostream &operator <<
   (std::ostream &o, const slvector<T>& v) { gmm::write(o,v); return o; }
 
-#ifdef USING_BROKEN_GCC295
-  template <class T> struct linalg_traits<const slvector<T> >
-    : public linalg_traits<slvector<T> > {};
-#endif
-
-  template <class T>
+  template <typename T>
   inline size_type nnz(const slvector<T>& l) { return l.last() - l.first(); }
 
 }
 
-#endif /* __GMM_VECTOR_H */
+namespace dal {
+
+  template <typename T, typename V> inline
+  T sqr(const gmm::ref_elt_vector<T, V> &a)
+  { return T(a) * T(a); }
+
+  template <typename T, typename V> inline
+  typename gmm::number_traits<T>::magnitude_type
+  abs(const gmm::ref_elt_vector<T, V> &a)
+  { return dal::abs(T(a)); }
+
+  template <typename T, typename V> inline
+  typename gmm::number_traits<T>::magnitude_type
+  abs_sqr(const gmm::ref_elt_vector<T, V> &a)
+  { return dal::abs_sqr(T(a)); }
+  template <typename T, typename V> inline
+  T conj(const gmm::ref_elt_vector<T, V> &a)
+  { return dal::conj(T(a)); }
+}
+
+namespace std {
+  template <typename T> void swap(gmm::wsvector<T> &v, gmm::wsvector<T> &w)
+  { v.swap(w);}
+  template <typename T> void swap(gmm::rsvector<T> &v, gmm::rsvector<T> &w)
+  { v.swap(w);}
+  template <typename T> void swap(gmm::slvector<T> &v, gmm::slvector<T> &w)
+  { v.swap(w);}
+}
+
+
+
+#endif /* GMM_VECTOR_H__ */
